//  Copyright Snap Inc. All rights reserved.
//  CameraKit

#import <Foundation/Foundation.h>
#import <AVFoundation/AVFoundation.h>
#import "SCCameraKitLensesConfig.h"

@protocol SCCameraKitLensesComponent;
@protocol SCCameraKitOutput;
@protocol SCCameraKitLensProcessor;
@protocol SCCameraKitLensHintDelegate;
@protocol SCCameraKitInput;
@protocol SCCameraKitARInput;
@protocol SCCameraKitErrorHandler;
@class SCCameraKitDataProviderComponent;

NS_ASSUME_NONNULL_BEGIN

NS_SWIFT_NAME(CameraKitProtocol)
@protocol SCCameraKitProtocol

/// Contains lenses related objects.
@property (strong, nonatomic, readonly) id<SCCameraKitLensesComponent> lenses;

/// Begin processing input frames with front camera position and portrait video orientation
/// @param input the input to configure.
/// @param arInput the ARKit input to configure.
- (void)startWithInput:(id<SCCameraKitInput>)input
               arInput:(id<SCCameraKitARInput>)arInput NS_SWIFT_NAME(start(input:arInput:));

/// Begin processing input frames.
/// @param input the input to configure.
/// @param arInput the ARKit input to configure.
/// @param cameraPosition the camera position in use
/// @param videoOrientation the orientation for the outputted video buffers
/// @param dataProvider data provider component to pass in custom data providers (optional -- will create and handle data providers by default if nil)
/// @param hintDelegate lens hint delegate to show/hide hints for applied lenses
- (void)startWithInput:(id<SCCameraKitInput>)input
               arInput:(id<SCCameraKitARInput>)arInput
        cameraPosition:(AVCaptureDevicePosition)cameraPosition
      videoOrientation:(AVCaptureVideoOrientation)videoOrientation
          dataProvider:(nullable SCCameraKitDataProviderComponent *)dataProvider
          hintDelegate:(nullable id<SCCameraKitLensHintDelegate>)hintDelegate NS_SWIFT_NAME(start(input:arInput:cameraPosition:videoOrientation:dataProvider:hintDelegate:));

/// End processing input frames.
- (void)stop;

/// Add an output. Frames will still be processed if no outputs exist.
- (void)addOutput:(id<SCCameraKitOutput>)output NS_SWIFT_NAME(add(output:));

/// Remove an output.
- (void)removeOutput:(id<SCCameraKitOutput>)listener NS_SWIFT_NAME(remove(output:));

/// The camera position in use. Setting will automatically update the input.
@property (nonatomic, assign) AVCaptureDevicePosition cameraPosition;

/// The orientation for the outputted video buffers
@property (nonatomic, assign) AVCaptureVideoOrientation videoOrientation;

/// The active input. May be either standard camera input or the AR input, depending on lens requirements.
@property (nonatomic, readonly) id<SCCameraKitInput> activeInput;

@end

NS_SWIFT_NAME(Session)
/// CameraKit handles interaction with the camera and contains several components like lenses.
@interface SCCameraKitSession : NSObject <SCCameraKitProtocol>

/// Init with lenses config instance to configure properties within lenses component
/// @param lensesConfig lenses config instance
/// @param errorHandler optional error handler instance to handle exceptions thrown by CameraKit
- (instancetype)initWithLensesConfig:(SCCameraKitLensesConfig *)lensesConfig
                        errorHandler:(nullable id<SCCameraKitErrorHandler>)errorHandler;

@end

NS_ASSUME_NONNULL_END
