import Foundation
import NIO
import Logging
import SwiftProtobuf
import NIOHTTP1
import GRPC
import os.log

protocol StreamFactory {
    func buildStream(service: TrillerSignalsSignalServiceClientProtocol,
                     authToken: String) -> BidirectionalStreamingCall<TrillerSignalsSignalRequest, TrillerSignalsStatus>
}

class GRPCSteamFactory: StreamFactory {

    func buildStream(service: TrillerSignalsSignalServiceClientProtocol,
                     authToken: String)
    -> BidirectionalStreamingCall<TrillerSignalsSignalRequest, TrillerSignalsStatus> {

        // Use CallOptions to send the auth token (necessary) and set a custom timeout (optional).
        var callOptions =  CallOptions()
        callOptions.customMetadata.add(name: "authorization",
                                       value: "\(authToken)")

        // Create/start the API call.
        let stream = service.signal(callOptions: callOptions, handler: { response in
            if response.code != 0 {
                os_log("Failed to send signal message %{PUBLIC}@",
                       log: NetworkCore.grpcCategory,
                       type: .debug, response.details)
            } else {
                os_log("Successfully recieved response code %{PUBLIC}@",
                       log: NetworkCore.grpcCategory,
                       type: .debug, response.code)
            }
        })
        return stream
    }

}

protocol GRPCManagerDelegate: AnyObject {
    func grpc(error: Error)
}

@objc
extension NetworkCore {
    internal class GRPCManager {
        weak var delegate: GRPCManagerDelegate?
        private static var instance: GRPCManager?

        class func shared(url: URL, authToken: String) -> GRPCManager? {
            if let instance = instance {
                // update manager
                instance.update(url: url,
                                authToken: authToken)
                return instance
            } else {
                instance = GRPCManager(url: url,
                                       authToken: authToken,
                                       worker: NetworkCore.eventLoopGroup)
                return instance
            }
        }

        private let streamFactory: StreamFactory
        internal let service: TrillerSignalsSignalServiceClient
        private var eventLoopGroup: EventLoopGroup
        private var stream: BidirectionalStreamingCall<TrillerSignalsSignalRequest, TrillerSignalsStatus>

        init?(url: URL, authToken: String, worker: EventLoopGroup, streamFactory: StreamFactory = GRPCSteamFactory()) {
            guard let port = url.port, let host = url.host else { return nil }
            let connection = ClientConnection.insecure(group: worker).connect(host: host, port: port)
            self.eventLoopGroup = worker
            self.service = TrillerSignalsSignalServiceClient(channel: connection)
            self.streamFactory = streamFactory
            self.stream = streamFactory.buildStream(service: service, authToken: authToken)
        }

        func update(url: URL, authToken: String, completion: (() -> Void)? = nil) {
            // we want to create a new stream due to auth token change
            _ = stream.sendEnd().always({ (result) in
                switch result {
                case .failure(let error):
                    os_log("Failed to end streem",
                           log: NetworkCore.grpcCategory,
                           type: .debug, error.localizedDescription)
                    self.delegate?.grpc(error: error)
                case .success:
                    self.stream = self.streamFactory.buildStream(service: self.service, authToken: authToken)
                }
                completion?()
            })
        }

        func makeRPCCall(_ request: TrillerSignalsSignalRequest) {

            _ = stream.sendMessage(request).always { (response) in
                switch response {
                case .failure(let error):
                    os_log("Failed to send signal for event %{PUBLIC}@",
                           log: NetworkCore.grpcCategory,
                           type: .debug, error.localizedDescription)
                    self.delegate?.grpc(error: error)
                case .success:
                    os_log("successfully sent signal for event %{PUBLIC}@",
                           log: NetworkCore.grpcCategory,
                           type: .debug, request.events.map({ $0.signalType }))
                }
            }
        }
    }
}
