import Foundation
import SwiftProtobuf
import Swift
import os.log
public protocol SignalsDelegate: AnyObject {
    func grpc(error: Error)
}
extension NetworkCore {

    @objc public class Signals: NSObject {
        weak public var delegate: SignalsDelegate?
        private static var instance: Signals?

        public class func shared(userId: Int64, authToken: String, url: URL) -> Signals? {
            if instance == nil {
                instance = Signals(userID: userId,
                                   authToken: authToken,
                                   url: url)
            } else if authToken != instance?.authToken {
                instance?.update(userID: userId,
                                 authToken: authToken,
                                 url: url)
            }
            return instance
        }

        private let queue = DispatchQueue(label: "co.triller.signals",
                                  qos: .background,
                                  autoreleaseFrequency: .workItem,
                                  target: nil)

        private let manager: GRPCManager?
        public typealias SignalRequest = (feed: TrillerSignalsFeed?,
                                          videoId: Int64,
                                          watchTime: Float,
                                          videoLength: Float,
                                          trackIsrc: String?,
                                          event: TrillerSignalsSignalType)

        @objc public enum EventSource: Int {
            public typealias RawValue = Int
            case undefinedFeed // = 0
            case followFeed // = 1
            case sharedFeed // = 2
            case socialFeed // = 3
            case videoFeed // = 4

            public var rawValue: Int {
                switch self {
                case .undefinedFeed: return 0
                case .followFeed: return 1
                case .sharedFeed: return 2
                case .socialFeed: return 3
                case .videoFeed: return 4
                }
            }
            public init?(rawValue: Int) {
                switch rawValue {
                case 1: self = .followFeed
                case 2: self = .sharedFeed
                case 3: self = .socialFeed
                case 4: self = .videoFeed
                default: self = .undefinedFeed
                }
            }
        }

        private var userID: Int64
        private var authToken: String

        public init(userID: Int64, authToken: String, url: URL) {
            self.userID = userID
            self.authToken = authToken
            manager = NetworkCore.GRPCManager.shared(url: url, authToken: authToken)
            super.init()
            manager?.delegate = self
            os_log("Initialised with authtoken: %{PRIVATE}@",
                   log: NetworkCore.grpcCategory,
                   type: .info, authToken)
        }

        internal init(userID: Int64, authToken: String, url: URL, manager: GRPCManager? = nil) {
            self.userID = userID
            self.authToken = authToken
            self.manager = manager ?? NetworkCore.GRPCManager.shared(url: url,
                                                                     authToken: authToken)
            super.init()

            os_log("Initialised with authtoken: %{PRIVATE}@",
                   log: NetworkCore.grpcCategory,
                   type: .info, authToken)
        }

        public func sendEvent(_ request: SignalRequest) {
            queue.async {
                self.sendSignal(request)
            }
        }
    }
}

private extension NetworkCore.Signals {
    func update(userID: Int64, authToken: String, url: URL) {
        self.userID = userID
        self.authToken = authToken
        manager?.update(url: url,
                        authToken: authToken)
    }
    func sendSignal(_ request: SignalRequest) {

        guard checkIfValid(videoId: request.videoId,
                           watchTime: request.watchTime,
                           videoLength: request.videoLength) else {

            return
        }

        let event = TrillerSignalsEvent.with { (data) in
            guard let feed = request.feed, let feedId = TrillerSignalsFeed.init(rawValue: feed.rawValue) else { return }
            data.feedID = feedId
            data.signalType = request.event
            data.videoID = request.videoId
            data.watchTime = request.watchTime
            data.videoLength = request.videoLength
            data.signalOccurredEpoch =  SwiftProtobuf.Google_Protobuf_Timestamp.init(date: Date())
        }

        let signalRequest = TrillerSignalsSignalRequest.with { [weak self] (data) in
            if let self = self {
                data.userID = self.userID
                data.events = [event]
            }
        }
        manager?.makeRPCCall(signalRequest)
    }

    func checkIfValid(videoId: Int64, watchTime: Float, videoLength: Float) -> Bool {
        guard videoLength > 0, videoId > 0 else {
            return false
        }
        return true
    }
}

extension NetworkCore.Signals: GRPCManagerDelegate {
    func grpc(error: Error) {
        delegate?.grpc(error: error)
    }
}

extension TrillerSignalsSignalType: CustomDebugStringConvertible {
    public var debugDescription: String {
        switch self {
        case .followedCreator: return "Followed creator"
        case .like: return "Like"
        case .shareIntention: return "Share"
        case .undefinedSignalType: return "Undefined"
        case .viewedCreatorProfile: return "Viewed profile"
        case .watchEvent: return "Watched"
        case .UNRECOGNIZED(let val): return "Signal: \(val)"
        case .directMessage: return "Direct Message"
        case .comment: return "Comment"
        }
    }
}
